import sys,time,string

try:
   # Try to import xmllib 0.5.1 or higher 
   from xml.parsers import xmllib
except:
   # import at least xmllib 0.2 which is part of Python 1.5.2
   # future versions of Zope should come with xmllib 0.5.1 or later
   import xmllib

"""
Given any XML File name as a string, XML2Zope assumes this
is File in a Zope Folder, and imports it into Zope.
Otherwise, XML2Zope expects a File from the file system, and
also imports it into Zope.

Uses: xmllib
      Look for this library at: http://www.python.org/sigs/xml-sig/

"""

import random

if __name__ != '__main__':
   from DocumentTemplate import html_quote

def XML2Zope(self, xml, REQUEST=None):
   import Globals
   import App.Dialogs

   obdir=self

   document_parser=XML2ZopeParser(obdir)

   print type(xml)

   if type(xml)==type(""):
      # so this must be a file name of a "File" in a Zope folder
      xml=str(getattr(obdir, xml))
      print type(xml)
      document_parser.parse_xml(xml)
   else:
      # So this is a file from the file system
      document_parser.parse_xml_file(xml)

   #Return an HTML message upon success
   title = "XML2Zope completed"
   message = title
   action= REQUEST['URL1']+'/manage_main'
   target = 'manage_main'
   return App.Dialogs.MessageDialog(title=title, message=message, action=action, target=target)

class Text_List:
   def __init__(self):
      self.text_list=[]

   def get_text(self):
      text_together=""
      for text in self.get_text_list():
         text_together=text_together+string.strip(text.get_text())
      return text_together

   def get_text_list(self):
      return self.text_list

   def add_data(self, data):
      text_list=self.get_text_list()
      text_list.append(self.create_DATA_Text(data))

   def add_cdata(self, cdata):
      text_list=self.get_text_list()
      text_list.append(self.create_CDATA_Text(cdata))

   def create_DATA_Text(self,data):
      return Text(data)

   def create_CDATA_Text(self,data):
      return self.create_DATA_Text(data)

class Zope_Text_List(Text_List):
   def create_DATA_Text(self,data):
      return Text(data)

   def create_CDATA_Text(self,data):
      return Zope_CDATA_Text(data)

class Text:
   def __init__(self,text):
      self.set_text(text)

   def set_text(self,text): 
      self.text=text

   def get_text(self):
      return self.text

class Zope_CDATA_Text(Text):
   def get_text(self):
      return html_quote(self.text)

class Tag_Stack:
   def __init__(self, tag ):
      self.stack=[]
      self.stack.append(tag)
   def push(self, tag):
      self.stack.append(tag)
   def pop(self):
      tag=self.stack[-1:]
      self.stack=self.stack[:-1]
      return tag[0]
   def top(self):
      tag=self.stack[-1:]
      return tag[0]

class Tag:
   def __init__(self, name, attributes):
      self.name = name
      self.text_list=self.create_text_list()
      self.attributes=attributes

   def create_text_list(self):
      return Text_List()

   def get_name(self):
      return self.name

   def get_text(self):
      return self.text_list.get_text()

   def add_data_to_text_list(self,data):
      self.text_list.add_data(data)

   def add_cdata_to_text_list(self,cdata):
      self.text_list.add_cdata(cdata)

   def has_attributes(self):
      return self.attributes!={}

   def get_attribute_by_name(self, name):
      return self.attributes[name]

class Zope_Tag(Tag):
   def __init__(self, name, attributes):
      self.name=""
      Tag.__init__(self,name,attributes)

   def create_text_list(self):
      return Zope_Text_List()

   def has_attributes(self):
      return self.attributes!={}

   def get_name(self):
      return self.name

class xmlParser(xmllib.XMLParser):
    def __init__(self):
        self.current_tag=self.get_new_tag("empty",{})
        self.tag_stack=Tag_Stack(self.current_tag)
        xmllib.XMLParser.__init__(self)

    def unknown_starttag(self,tag,attrs):
        self.hook_tag(tag,attrs)

    def unknown_endtag(self, tag):
        self.leave_hook()
    
    def handle_data(self, data):
        if self.no_whitespace(data):
           self.current_tag.add_data_to_text_list(data)

    def handle_cdata(self, cdata):
        if self.no_whitespace(cdata):
           self.current_tag.add_cdata_to_text_list(cdata)

    def no_whitespace(self,text):
        no_whitespace=0
        if string.strip(text)=="":
           no_whitespace=0
        else:
           no_whitespace=1
        return no_whitespace

    def close(self):
        xmllib.XMLParser.close(self)

    # hooks a new sub_tag to a tag and moves down to it
    def hook_tag ( self, tag_name, attributes):
        print "Hooking", tag_name
        tag = self.get_new_tag(tag_name, attributes)
        self.current_tag = tag
        self.tag_stack.push(self.current_tag)

    def get_new_tag(self,tag_name,attributes):
        return Tag(tag_name, attributes)
        
    # steps up from a hook
    def leave_hook( self):
        print self.current_tag.get_text()
        self.tag_stack.pop()
        self.current_tag = self.tag_stack.top()

    def parse_xml(self, xml_data):
        print "parse_xml", type(xml_data)
        self.feed(xml_data)
        self.close()

    def parse_xml_file(self, xml_file):
        print "parse_xml_file", type(xml_file)
        while 1:
           line = xml_file.readline()
           if line:
              self.feed(line)
           else:
              break
        self.close()


class XML2ZopeParser(xmlParser):
    def __init__(self, obdir):
        self.current_obdir=obdir
        xmlParser.__init__(self)

    # hooks a new sub_tag to a tag and moves down to it
    def hook_tag ( self, tag_name, attributes):
        xmlParser.hook_tag(self, tag_name, attributes)
        self.create_folder(self.current_tag)

    # steps up from a hook
    def leave_hook(self):
        self.create_attributes()
        print "My parents id is:", self.current_obdir.aq_parent.id
        print "My parent is:",self.current_obdir.aq_parent
        self.current_obdir=self.current_obdir.aq_parent
        xmlParser.leave_hook(self)
 
    def create_folder(self,tag):
        father=self.current_obdir
        print "Creating Folder: " + tag.get_name()
        folder_name=tag.get_name() + "_" + self.get_random_int(father,tag.get_name())
        print folder_name
        father.manage_addFolder(folder_name)
        self.current_obdir=getattr(father,folder_name)
        self.current_obdir.manage_changeProperties(title=tag.get_name())

    def get_random_int(self, obdir,name):
        #quick hack !
        random_int=str(random.randint(1,32768))
        while hasattr(obdir, name+"_"+random_int):
           random_int=random.randint(1,32768)
        return random_int

    def create_attributes(self):
        print self.current_tag
        print self.current_tag.name
        print self.current_tag.text_list
        print self.current_tag.attributes
        for attribute in self.current_tag.attributes.items():
           id, value=attribute
           self.create_attribute(id, value)
        self.current_obdir.manage_addProperty("data", self.current_tag.get_text(), 'text')

    def create_attribute(self, id, value):
        print "Creating Attribute as Property: " + id + " of type: text" + " with value: " + value
        if id !="title" and id!="id":
           self.current_obdir.manage_addProperty(id, value, 'text')

    def get_new_tag(self,tag_name,attributes):
        return Zope_Tag(tag_name, attributes)

